import itertools

from pathlib import Path

from bomisspell.exceptions import HomophoneNotFound
from bomisspell.mingzhi import get_mingzhi_options
from bomisspell.sgnon_jung import get_sngon_jug_options
from bomisspell.utils import build_syl, get_syls, parse_syl, from_yaml
from bomisspell.yang_jug import get_yang_jug_options

def get_misspelled_opt(syl_parts, mingzhi_mapping={}):
    """Return all the combination of misspelled syllable by shuffeling sngonjug, yangjug and replacing mingzhi by its similar pronounciation

    Args:
        syl_parts (dict): consit of all the possible component of a syllable which are sngonjug, mingzhi, jesjug and yangjug

    Returns:
        list: misspelled options of the syllable
    """
    if not mingzhi_mapping:
        mingzhi_mapping_path  = Path(__file__).parent / "resources/mingzhi_mapping.yaml"
        mingzhi_mapping = from_yaml(mingzhi_mapping_path.read_text(encoding='utf-8'))
    options = []
    options += get_sngon_jug_options(syl_parts)
    options += get_mingzhi_options(syl_parts, mingzhi_mapping)
    options += get_yang_jug_options(syl_parts)
    options.append(build_syl(syl_parts))
    return options

def shuffel_syl(misspelled_syls):
    misspelled_words = []
    for misspelled_option in list(itertools.product(*misspelled_syls)):
        misspelled_words.append("".join(misspelled_option))
    return misspelled_words

def get_misspelled_word(word, mingzhi_mapping = {}):
    misspelled_syls = []
    misspelled_words = []
    if not mingzhi_mapping:
        mingzhi_mapping_path  = Path(__file__).parent / "resources/mingzhi_mapping.yaml"
        mingzhi_mapping = from_yaml(mingzhi_mapping_path.read_text(encoding='utf-8'))
    syls = get_syls(word)
    for syl in syls:
        syl_parts = parse_syl(syl)
        misspelled_syls.append(get_misspelled_opt(syl_parts, mingzhi_mapping))
    if misspelled_syls:
        misspelled_words = shuffel_syl(misspelled_syls)
    else:
        misspelled_words.append(word)
    misspelled_words.remove(word)
    if misspelled_words:
        return misspelled_words
    else:
        raise HomophoneNotFound



from pathlib import Path
from bomisspell.utils import from_yaml

def parse_mingzhi(mingzhi):
    """Parse mingzhi char and vowel

    Args:
        mingzhi (word): mingzhi char

    Returns:
        str: mingzhi root char, vowel
    """
    for vowel in ['ི', 'ུ', 'ེ', 'ོ']:
        if vowel in mingzhi:
            mingzhi = mingzhi.replace(vowel, '')
            return mingzhi, vowel
    return mingzhi, ''

def get_mingzhi_options(syl_parts, mingzhi_mapping={}):
    """Generate misspelled option by substituting similar sound mingzhi

    Args:
        syl_parts (dict): consit of all the possible component of a syllable which are sngonjug, mingzhi, jesjug and yangjug
        mingzhi_mapping (dict): mingzhi and its similar sound mingzhi

    Returns:
        list: misspelled options generated by substituting similar sound mingzhi
    """
    if not mingzhi_mapping:
        mingzhi_mapping_path  = Path(__file__).parent / "resources/mingzhi_mapping.yaml"
        mingzhi_mapping = from_yaml(mingzhi_mapping_path.read_text(encoding='utf-8'))
    options = []
    cur_sngon_jug = syl_parts.get('sngon_jug', '')
    cur_mingzhi, mingzhi_vowel = parse_mingzhi(syl_parts.get('mingzhi', ''))
    cur_jes_jug = syl_parts.get('jes_jug', '')
    cur_yang_jug = syl_parts.get('yang_jug', '')
    if cur_mingzhi in mingzhi_mapping:
        for mingzhi_opt in mingzhi_mapping[cur_mingzhi]:
            cur_option = f'{cur_sngon_jug}{mingzhi_opt}{mingzhi_vowel}{cur_jes_jug}{cur_yang_jug}་'
            options.append(cur_option)
    return options



from bomisspell.utils import build_syl

def get_yang_jug_options(syl_parts):
    """Return different combination yangjug on syllable

    Args:
        syl_parts (dict): consit of all the possible component of a syllable which are sngonjug, mingzhi, jesjug and yangjug

    Returns:
        list: combinations that can be obtain using yangjug
    """
    options = []
    sngon_jug = syl_parts.get('sngon_jug', '')
    mingzhi = syl_parts.get('mingzhi', '')
    jes_jug = syl_parts.get('jes_jug', '')
    yang_jug = syl_parts.get('yang_jug', '')
    if not jes_jug:
        return options
    syl_parts['yang_jug'] = ''
    if yang_jug:
        if yang_jug == 'ད':
            options.append(f'{sngon_jug}{mingzhi}{jes_jug}་')
            options.append(f'{sngon_jug}{mingzhi}{jes_jug}ས་')
        else:
            options.append(f'{sngon_jug}{mingzhi}{jes_jug}་')
            options.append(f'{sngon_jug}{mingzhi}{jes_jug}ད་')
    else:
        options.append(f'{sngon_jug}{mingzhi}{jes_jug}ས་')
        options.append(f'{sngon_jug}{mingzhi}{jes_jug}ད་')
    return options  



import re

from pathlib import Path
from bomisspell.utils import to_yaml

def parse_options(options_pat):
    options = []
    if options_pat:
        options = re.split(',', options_pat)
    return options

def parse_line(line):
    mingzhi = re.search('(\S+):', line).group(1)
    options_pat = re.search('\[(.*?)\]', line).group(1)
    options = parse_options(options_pat)
    return mingzhi, options

def create_mapping(mapping):
    mingzhi_mapping = {}
    lines = mapping.splitlines()
    for line in lines:
        print(line)
        mingzhi, options = parse_line(line)
        mingzhi_mapping[mingzhi] = options
    return mingzhi_mapping


if __name__ == "__main__":
    mapping = Path('./mapping.txt').read_text(encoding='utf-8')
    mingzhi_mapping = create_mapping(mapping)
    mingzhi_mapping = to_yaml(mingzhi_mapping, sort_keys=False, allow_unicode=True)
    Path('./mingzhi_mapping.yaml').write_text(mingzhi_mapping, encoding='utf-8')


from bomisspell.utils import build_syl

def get_sngon_jug_options(syl_parts):
    """Return different combination sngonjug on syllable

    Args:
        syl_parts (dict): consit of all the possible component of a syllable which are sngonjug, mingzhi, jesjug and yangjug

    Returns:
        list: combinations that can be obtain using sngonjug
    """
    options = []
    sngon_jug =  syl_parts.get('sngon_jug', "")
    mingzhi = syl_parts.get('mingzhi', '')
    jes_jug = syl_parts.get('jes_jug', '')
    yang_jug = syl_parts.get('yang_jug', '')
    if len(build_syl(syl_parts)) == 2:
        return options
    if sngon_jug:
        options.append(f'{mingzhi}{jes_jug}{yang_jug}་')
    for cur_sngon_jug in ['ག', 'ད', 'བ', 'མ', 'འ']:
        if cur_sngon_jug == sngon_jug:
            continue
        options.append(f'{cur_sngon_jug}{mingzhi}{jes_jug}{yang_jug}་')
    return options


import re
import pylibyaml
import yaml
from botok.textunits.sylcomponents import SylComponents

def to_yaml(data):
    return yaml.dump(data, sort_keys=False, allow_unicode=True, Dumper=yaml.CSafeDumper)
    
def from_yaml(data):
    return yaml.load(data, Loader=yaml.CSafeLoader)

def get_syls(word):
    """Return syllable of a word

    Args:
        word (str): word

    Returns:
        list: syllable of the words
    """
    chunks = re.split('་', word)
    syls= []
    for chunk in chunks:
        if chunk:
            syls.append(chunk)
    return syls

def is_consonant(char):
    """Check if the character is tibetan consonant or not

    Args:
        char (str): a tibetan character

    Returns:
        boolean: true if char is tibetan consonant else false
    """
    if re.search('[\u0F40-\u0F67]', char):
        return True
    else:
        return False

def is_vowel(char):
    """Check if the character is tibetan vowel or not

    Args:
        char (str): a character

    Returns:
        Boolean: True if char is vowel else false
    """
    if char in ['ི', 'ུ', 'ེ', 'ོ']:
        return True
    else:
        return False

def get_sngon_jug(syl_part):
    """Returns if sngon jug exist else empty string

    Args:
        syl_part (str): syllable component

    Returns:
        str: one the sngon jug if one exist in syl component else empty string 
    """
    if syl_part[0] in ['ག', 'ད', 'བ', 'མ', 'འ'] and is_consonant(syl_part[1]):
        return syl_part[0]
    else:
        return ''

def get_mingzhi(syl_first_half, syl_second_half, sngon_jug):
    """Return mingzhi of the syllable using syllable components

    Args:
        syl_first_half (str): first half of syl component which could consist of sngonjug and mingzhi
        syl_second_half (str): second half of syl component which could consist of vowel of mingzhi, jes jug and yang jug

    Returns:
        str: mingzhi of syllable
    """
    mingzhi = ''
    if sngon_jug:
        mingzhi += syl_first_half[1:]
    else:
        mingzhi += syl_first_half
    if syl_second_half and is_vowel(syl_second_half[0]):
        return mingzhi+syl_second_half[0]
    else:
        return mingzhi

def get_jes_jug(syl_part):
    """Return jes jug from the second half of syllable component if it exist

    Args:
        syl_part (str): second half of syllable component

    Returns:
        str: jes jug if it exist else empty string
    """
    if re.search('ག|ང|ད|ན|བ|མ|འ|ར|ལ|ས', syl_part):
        jes_jug_pat = re.search('ག|ང|ད|ན|བ|མ|འ|ར|ལ|ས', syl_part)
        return jes_jug_pat[0]
    else:
        if len(syl_part)>=2:
            return syl_part[1]
        else:
            return ''

def get_yang_jug(syl_part):
    """Return yang jug from second half of syllable component

    Args:
        syl_part (str): second half of syllable component

    Returns:
        str: yangjug if oen exist else empty string
    """
    if is_vowel(syl_part[0]):
        if len(syl_part)==3:
            return syl_part[2]
        else:
            return ""
    else:
        if len(syl_part)>0:
            return syl_part[1]
        else:
            return ""

def parse_syl(syl):
    """Parse syllable components

    Args:
        syl (str): syllable

    Returns:
        dict: consit of all the possible component of a syllable which are sngonjug, mingzhi, jesjug and yangjug
    """
    syl_parts = {}
    sngon_jug = ''
    jes_jug = ''
    mingzhi = ''
    yang_jug = ''
    sc = SylComponents()
    components = sc.get_parts(syl)
    if components:
        if len(components[0]) >= 2:
            sngon_jug = get_sngon_jug(components[0])
        mingzhi = get_mingzhi(components[0], components[1], sngon_jug)
        if components[1]:
            jes_jug = get_jes_jug(components[1])
        if len(components[1]) >= 2:
            yang_jug = get_yang_jug(components[1])
        syl_parts = {
            'sngon_jug': sngon_jug,
            'mingzhi': mingzhi,
            'jes_jug': jes_jug,
            'yang_jug': yang_jug
        }
    return syl_parts

def build_syl(syl_parts):
    sngon_jug = syl_parts.get('sngon_jug', '')
    mingzhi = syl_parts.get('mingzhi', '')
    jes_jug = syl_parts.get('jes_jug', '')
    yang_jug = syl_parts.get('yang_jug', '')
    syl = f'{sngon_jug}{mingzhi}{jes_jug}{yang_jug}་'
    return syl


import re
import setuptools
from pathlib import Path

with open("README.md", "r") as fh:
    long_description = fh.read()

def get_version(prop, project):
    project = Path(__file__).parent / project / "__init__.py"
    result = re.search(
        r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop), project.read_text()
    )
    return result.group(1)


setuptools.setup(
    name="bomisspell",
    version=get_version("__version__", "bomisspell"),
    author="Tenzin Kaldan",
    author_email="kaldantenzin@gmail.com",
    description="Generates misspelled tibetan word of a given word",
    py_modules=["bomisspell"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="Apache2",
    url="https://github.com/potala-dev/bomisspell",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_data={
        "bomisspell": [
            "resources/*",
        ]
    },
    install_requires=[
        "botok>=0.8.6, <8.0",
        "pylibyaml>=0.1.0, <2.0"
    ],
    python_requires=">=3.6",
    tests_require=["pytest"],
)
